
USE [Directions2010]  -- change db name
SET STATISTICS IO ON
--SET STATISTICS TIME OFF
GO

/* EXAMPLE 1 */

-- SP: Stmt Completed
SELECT  * FROM "Directions2010"."dbo"."Directions 2010$Document Dimension" WITH (READUNCOMMITTED)   
WHERE (("Table ID"=@P1 OR "Table ID"=@P2)) 
AND (("Document Type"=@P3)) AND (("Document No_"=@P4)) 
ORDER BY "Table ID","Document Type","Document No_","Line No_","Dimension Code" 


-- SP: RPC Completed
exec sp_cursorexecute 1073752977,@p2 output,@p3 output,@p4 output,@p5 output,36,37,1,'100006698'


-- Combined (ad-hoc)
SELECT  * FROM "Directions2010"."dbo"."Directions 2010$Document Dimension" WITH (READUNCOMMITTED)   
WHERE (("Table ID"=36 OR "Table ID"=37)) 
AND (("Document Type"=1)) AND (("Document No_"='100006698')) 
ORDER BY "Table ID","Document Type","Document No_","Line No_","Dimension Code" 

----------------------------------------------------------------------------------------------------

/* EXAMPLE 2 */

-- Combined (ad-hoc)
SELECT SUM("Quantity"),SUM("Invoiced Quantity"),SUM("Remaining Quantity") 
FROM "Directions2010"."dbo"."Directions 2010$Item Ledger Entry" 
WITH (READUNCOMMITTED)   
WHERE (("Item No_"='1906-S')) AND (("Variant Code"='')) AND (("Location Code"='YELLOW'))


-- supposedly optimized index (NAV compatible)
CREATE INDEX ssi_1 on "Directions 2010$Item Ledger Entry"
("Item No_", "Location Code", "Variant Code")


/* Basic Rules for Index Creation */
/*
- NEVER name customized SQL site indexes "$[Number]" - use own naming-convention apart from NAV's
- Equality Columns (=) first, then In-Equality (<, >, <>, LIKE) Columns
- Highly selective fields first
- Caution: Regard NAV "Key" context = ORDER BY clause
  ->  "Fast Forward Cursors" are optimizing for the WHERE CLAUSE
  ->  "Dynamic Cursors" are optimizing for the ORDER BY
  Thus, SQL Theory and NAV Reality are not always the same!
*/


-- supposedly even better optimized index (not NAV compatible)
CREATE INDEX ssi_2 on "Directions 2010$Item Ledger Entry"
("Item No_", "Location Code", "Variant Code")
INCLUDE
("Quantity", "Invoiced Quantity", "Remaining Quantity")


----------------------------------------------------------------------------------------------------


-- verify index usage
SELECT object_name(IDX."object_id") AS "object", IDX."name",
       IDXUSG."user_seeks", IDXUSG."user_scans", IDXUSG."user_lookups", IDXUSG."user_updates" 
FROM sys.dm_db_index_usage_stats IDXUSG
  JOIN sys.indexes IDX ON IDX."object_id" = IDXUSG."object_id" AND IDX."index_id" = IDXUSG."index_id"
WHERE IDXUSG."database_id" = db_id()
  AND IDX."name" LIKE 'ssi%' -- regard your naming convention
ORDER BY object_name(IDX."object_id"), IDX."name"


----------------------------------------------------------------------------------------------------

-- CleanUp
DROP INDEX ssi_1 on "Directions 2010$Item Ledger Entry"
DROP INDEX ssi_2 on "Directions 2010$Item Ledger Entry"

----------------------------------------------------------------------------------------------------
