
USE [NAV]  -- change db name
GO

-- Create table to store Index Usage statistics

CREATE TABLE [dbo].[ssi_IndexUsage_Tab]
(
[timestamp] datetime,
[database_id] smallint,
[object_id] int,
[index_id] int,
[user_seeks] bigint,
[user_scans] bigint,
[user_lookups] bigint,
[user_updates] bigint,
[last_user_seek] datetime,
[last_user_scan] datetime,
[last_user_lookup] datetime,
[last_user_update] datetime,
[system_seeks] bigint,
[system_scans] bigint,
[system_lookups] bigint,
[system_updates] bigint,
[last_system_seek] datetime,
[last_system_scan] datetime,
[last_system_lookup] datetime,
[last_system_update] datetime
CONSTRAINT [ssi_IndexUsage_Tab$pk_ci] PRIMARY KEY CLUSTERED ([timestamp],[database_id],[object_id],[index_id])
)
GO

----------------------------------------------------------------------------------------------------

-- Create SQL Agent Job to periodically (e.g. 2 weekly) copy DMV

INSERT INTO dbo.[ssi_IndexUsage_Tab]
  SELECT getdate(), * FROM sys.dm_db_index_usage_stats 
  WHERE [database_id] = db_id()
  ORDER BY object_name([object_id]), [index_id]
GO
-- CleanUp/Reset:  TRUNCATE TABLE dbo.[ssi_IndexUsage_Tab]

----------------------------------------------------------------------------------------------------

-- Verify long-term Index Usage statistics

SELECT
    db_name(USG.[database_id]) as [database_name]
   ,USG.[object_id]
   ,object_name(USG.[object_id]) as [object_name]
   ,USG.[index_id]
   ,max(IDX.[name]) as [index_name]
   ,max(USG.[user_seeks]) as [user_seeks]
   ,max(USG.[user_scans]) as [user_scans]
   ,max(USG.[user_lookups]) as [user_lookups]
   ,max(USG.[user_updates]) as [user_updates]
   ,max(USG.[last_user_seek]) as [last_user_seek]
   ,max(USG.[last_user_scan]) as [last_user_scan]
   ,max(USG.[last_user_lookup]) as [last_user_lookup]
   ,max(USG.[last_user_update]) as [last_user_update]
   ,max(USG.[system_seeks]) as [system_seeks]
   ,max(USG.[system_scans]) as [system_scans]
   ,max(USG.[system_lookups]) as [system_lookups]
   ,max(USG.[system_updates]) as [system_updates]
   ,max(USG.[last_system_seek]) as [last_system_seek]
   ,max(USG.[last_system_scan]) as [last_system_scan]
   ,max(USG.[last_system_lookup]) as [last_system_lookup]
   ,max(USG.[last_system_update]) as [last_system_update]
  FROM dbo.[ssi_IndexUsage_Tab] as USG
  INNER JOIN sys.indexes as IDX on USG.[object_id] = IDX.[object_id] and USG.[index_id] = IDX.[index_id]
  WHERE ((objectproperty(USG.[object_id], N'IsUserTable') = 1) or (objectproperty(USG.[object_id], N'IsView') = 1))
    AND (INDEXPROPERTY (USG.[object_id], IDX.[name], N'IsClustered') = 0) 
  GROUP BY USG.[database_id], USG.[object_id], USG.[index_id]
  HAVING
    -- Example:
    -- change thresholds/filters as required
    max(USG.[user_seeks]) = 0 AND   
    max(USG.[user_scans]) = 0 AND  
    max(USG.[user_lookups]) = 0 AND
    max(USG.[user_updates]) >= 1000 -- change threshold 
  ORDER BY object_name(USG.[object_id]), USG.[index_id]
GO

----------------------------------------------------------------------------------------------------
